#################################################################################
# Created       : Martin Foster
# Modified      : 04/21/2002
#################################################################################
#
# Database - Modules part of Webchat designed to aid Database programming  
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package Ethereal::Database;					# Package name

use DBI;							# Database indepedent
use strict;							# Strict variable enforcement

#################################################################################
# Data Members
#################################################################################

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
        my $class = shift;
        my $self  = { };
        
	# Bless $self into designated class
	bless $self, $class;

	# Return self	
        return $self;
}

#####################
# Database Connect
#
# Sub-routine to allow connection to the database while revealing as little as
# possible to the coder.  Also centralized

sub Connect
{
	#####################
	# Data members

	my $self     = shift;					# Self reference
	my $database;				 		# Database handle

	# User variables
	my $db		= 'webchat';				# Database
	my $server	= 'localhost';				# Server to connect
	my $port	= '3306';				# Port to use
	my $login	= 'webchat';				# Login name
	my $pass	= 'CHANGE';				# Password
	

	#####################
	# Program area

	# Establish connection
	$database = DBI->connect("DBI:mysql:$db:$server:$port",
				$login,
				$pass,
				{PrintError => 1})
	|| die ("Database error \#$DBI::err :  $DBI::errstr");


	# Store handle
	$self->{'HANDLE'} = $database;
}


#####################
# Document Get
#
# Data retreival sub-routine for WebchatText table in Webchat database

sub DocumentGet	
{
	#####################
	# Data members

	my $self     = shift;					# Referene to self
	my $field    = shift;					# Search field

	my $data;						# Data pulled from database
	my $database = $self->{'HANDLE'};			# Database handle
	my $statement;						# SQL statement


	#####################
	# Program area

	# Initialize and execute
	$statement = $database->prepare("SELECT WebchatTextContent 
		FROM  WebchatText 
		WHERE WebchatTextName=?");
	$statement->execute($field);

	# Retreive and finish
	$data = $statement->fetchrow();
	$statement->finish;

	# Return gathered information
	return $data;
}
 
# Alias for Acceptable Use Policy
sub DocumentGetAUP
{ print DocumentGet(shift, 'AcceptableUsePolicy'); }

# Board notice
sub DocumentGetBoardNotice
{ return DocumentGet(shift, 'BoardNotice'); }

# Donations section
sub DocumentGetDonations
{ return DocumentGet(shift, 'WebDonation'); }

# Donations Mail
sub DocumentGetDonationsMail
{ return DocumentGet(shift, 'MailDonation'); }

# Alias for Header
sub DocumentGetHeader
{ print DocumentGet(shift, 'WebHeader'); }

# Alias for Header
sub DocumentGetHeaderPlain
{ print DocumentGet(shift, 'WebHeaderPlain'); }

# Alias for Home Header
sub DocumentGetHomeHeader
{ return DocumentGet(shift, 'HomeHeader'); }

# Alias for Home Readme
sub DocumentGetHomeReadme
{ return DocumentGet(shift, 'HomeReadme'); }

# Alias for Footer
sub DocumentGetFooter
{ print DocumentGet(shift, 'WebFooter'); }

# Alias for Invitation
sub DocumentGetInvitation
{ return DocumentGet(shift, 'MailInvitation'); }

# Mail notice
sub DocumentGetMailNotice
{ return DocumentGet(shift, 'MailNotice'); }

# Mail posts notificiation and instructions
sub DocumentGetMailPosts
{ return DocumentGet(shift, 'MailPosts'); }

# Mail confirmed
sub DocumentGetConfirmed
{ return DocumentGet(shift, 'MailVerified'); }

# Alias for Registration
sub DocumentGetRegistration
{ print DocumentGet(shift, 'Registration'); }

# Alias for Accepted registration
sub DocumentGetAccepted
{ print DocumentGet(shift, 'Accepted'); }

# Alias for Javascript
sub DocumentGetJavascript
{ print DocumentGet(shift, 'WebJavascript'); }


#####################
# Fetch Realm
#
# Used to retrieve the information from realms, making first sure that they
# have indeed made use of the functionality.  Otherwise things, will tend
# to falter.

sub FetchRealm
{
	#####################
	# Data members

	my $self     = shift;					# Referene to self
	my $field    = shift;					# Search field
	my $realm    = shift;					# Affected realm

	my $value;						# Value pulled from database


	#####################
	# Program area

	# Initial retrieval
	($value) = $self->DataGet("SELECT DesignContent
		FROM  RealmDesign
		WHERE RealmName=?
		  AND WebchatTextName=?", $realm, $field);


	# If no value exists
	unless (defined($value))
	{
		# Retreive defaults
		$value = $self->DocumentGet($field);
	}

	# Return value for use
	return $value;
}

# Alias for realm banner
sub FetchRealmBanner
{ 
	#####################
	# Data members

	my $self     = shift;					# Referene to self

	my $banner;						# Retreived banned
	my $data;						# Fortune data
	my $idnt;						# Fortune type


	#####################
	# Program area
	
	# Fetch banner
	$banner = FetchRealm($self, 'RealmBanner', shift); 

	
	# Verify for fortune
	if ($banner =~ /FRTN([\w\-]+)/s)
	{
		# Force case
		$idnt = lc($1);

		# Retreive data
		($data) = DataGetFortune($self, $idnt);


		# Replace if defined
		if (defined($data))
		{
			# Use $1 for proper case
			$banner =~ s/FRTN$1/$data/gs;
		}
	}


	# Return banner
	return $banner;
}

# Alias for dice roller
sub FetchRealmDice
{ return FetchRealm(shift, 'RealmDice', shift); }

# Alias for footer
sub FetchRealmFooter
{ return FetchRealm(shift, 'RealmFooter', shift); }

# Alias for header 
sub FetchRealmHeader
{ return FetchRealm(shift, 'RealmHeader', shift); }

# Alias for information
sub FetchRealmInfo
{ return FetchRealm(shift, 'RealmInfo', shift); }

# Alias for panel
sub FetchRealmPanel
{ return FetchRealm(shift, 'RealmPanel', shift); }

# Alias for private messages
sub FetchRealmPrivate
{ return FetchRealm(shift, 'RealmPrivate', shift); }

# Alias for auto-scrolling remote control
sub FetchRealmRemote
{ return FetchRealm(shift, 'RealmRemote', shift); }

# Alias for seperator bar
sub FetchRealmSeperator
{ return FetchRealm(shift, 'RealmSeperator', shift); }


#####################
# Get Hashes
#
# Sub-routine created to take a reference to an array and retrieve a list of
# all records within a specific field of information.

sub GetHash 
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $hash     = shift;					# Reference of hash
	my $type     = shift;					# Type of values

	my $database = $self->{'HANDLE'};			# Database handle
	my $statement;						# SQL Statement

	my $key;						# Key of hash
	my $value;						# Value for hash


	#####################
	# Program area

	# Prepare and execute
	$statement = $database->prepare("SELECT 
		 TagName, 
		 TagData
		FROM  Tag
		WHERE TagType=?");
	$statement->execute($type);

	# Retreive and use
	while (($key, $value) = $statement->fetchrow())
	{ $hash->{$key} = $value; }

	# Finish statement
	$statement->finish();
}

# Alias for Board hash
sub GetHashBoard
{ GetHash(shift, shift, 'board'); }

# Alias for Configuration hash
sub GetHashConfiguration
{ GetHash(shift, shift, 'configuration'); }

# Alias for Chat hash
sub GetHashHome
{ GetHash(shift, shift, 'home'); }

# Alias for Login hash
sub GetHashLogin
{ GetHash(shift, shift, 'login'); }

# Alias for Mail hash
sub GetHashMail
{ GetHash(shift, shift, 'mail'); }

# Alias for Portal hash
sub GetHashPortal
{ GetHash(shift, shift, 'portal'); }

# Alias for Realm hash
sub GetHashRealm
{ GetHash(shift, shift, 'realm'); }

# Alias for System hash
sub GetHashSystem
{ GetHash(shift, shift, 'system'); }


#####################
# GetList
#
# Sub-routine created to take a reference to an array and retrieve a list of
# all records within a specific field of information.

sub GetList                                                     # Database handle, reference to array, SQL
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $array    = shift;					# Reference of array
	my $query    = shift;					# SQL query

	my $database = $self->{'HANDLE'};			# Database handle
	my $statement;						# SQL Statement


	#####################
	# Program area

	# Prepare and execute
	$statement = $database->prepare($query);
	$statement->execute(@_);

	# Cleanup of array
	splice(@{ $array }, 0);

	# Retrieve and place in array
	while ($_ = $statement->fetchrow())
	{ push(@{ $array }, $_); }

        # End statement
        $statement->finish;
}

# Alias for designs
sub GetListDesign
{ GetList(shift, shift, "SELECT WebchatTextName FROM WebchatText WHERE WebchatTextName LIKE 'Realm%'"); }

# Alias for documents
sub GetListDocuments
{ GetList(shift, shift, 'SELECT WebchatTextName FROM WebchatText ORDER BY WebchatTextName'); }

# Alias for guest realms
sub GetListGuestRealms
{ GetList(shift, shift, "SELECT RealmName FROM Realm WHERE RealmPublic='yes' AND RealmGuests='yes' ORDER BY RealmName"); }

# Alias for public realms
sub GetListPublicRealms
{ GetList(shift, shift, "SELECT RealmName FROM Realm WHERE RealmPublic='yes' ORDER BY RealmName"); }

# Alias for private realms
sub GetListSupervisorRealms
{ GetList(shift, shift, "SELECT RealmName FROM RealmSupervisor WHERE PuppeteerLogin=? ORDER BY RealmName", @_); }

# Alias for non-private realms
sub GetListNonPrivate
{ GetList(shift, shift, "SELECT RealmName FROM Realm WHERE RealmPublic='yes' OR RealmPublic='hybrid' ORDER BY RealmName"); }

# Alias for private realms
sub GetListGlobalRealms
{ GetList(shift, shift, "SELECT RealmName FROM Realm WHERE RealmPublic <> 'alias' ORDER BY RealmName"); }

# Alias for puppet listing that are yours
sub GetListPuppetYours
{ GetList(shift, shift, "SELECT PuppetName FROM Puppet WHERE PuppetPool=? AND PuppeteerLogin=? AND PuppetActive IS NULL ORDER BY PuppetName", 'system', @_); }

# System and Realm handles
sub GetListPuppetRealm
{ GetList(shift, shift, "SELECT PuppetName FROM Puppet WHERE PuppeteerLogin=? AND PuppetActive IS NULL AND (PuppetPool=? OR PuppetPool=?) ORDER BY PuppetName", shift, 'system', @_);}

# Alias for puppet listing that are yours
sub GetListPuppetIgnored
{ GetList(shift, shift, "SELECT PuppetName FROM PuppetIgnore WHERE PuppeteerLogin=? ORDER BY PuppetName", @_); }

# Alias for puppet listing that are yours
sub GetListPuppetSuspended
{ GetList(shift, shift, "SELECT PuppetName FROM Puppet WHERE PuppeteerLogin=? AND PuppetActive IS NOT NULL ORDER BY PuppetName", @_); }

# Alias for EMail retreival
sub GetListEMails
{ GetList(shift, shift, "SELECT p.PuppeteerEMail FROM Puppeteer p, PuppeteerRegistration r WHERE p.PuppeteerLogin <> r.PuppeteerLogin GROUP BY PuppeteerEMail ORDER BY PuppeteerEMail"); }

# Alias for who is online
sub GetListWho
{ GetList(shift, shift, "SELECT DISTINCT(po.PuppetName) FROM Post po, Puppet pu WHERE pu.PuppeteerLogin=po.PuppeteerLogin AND pu.PuppetPool=? AND po.RealmName=? AND po.PostTimestamp > ? ORDER BY po.PuppetName", 'system', @_); }

# Alias for mods
sub GetListMods
{ GetList(shift, shift, "SELECT PuppeteerLogin FROM Puppeteer WHERE PuppeteerModPrivs='yes' ORDER BY PuppeteerLogin", @_); } 

# Restrictions
# Alias for restricted words
sub GetListWords
{ GetList(shift, shift, "SELECT RestrictionValue FROM Restriction WHERE RealmName=? AND RestrictionType='word'", @_); }

sub GetListImageBan
{ GetList(shift, shift, "SELECT RestrictionValue FROM Restriction WHERE RealmName='system' AND RestrictionType='picture'", @_); }


#####################
# GetData
#
# Generic sub-routine that will return a single row of data be it in multiple
# amounts or just one field within the row there is a special section for
# font handling if choice is at all defined.

sub DataGet
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $query    = shift;					# Database query

	my $database = $self->{'HANDLE'};			# Database get
	my $statement;						# SQL Statement

	my @values;						# Values returned from table


	#####################
	# Program area

	# Prepare and execute
	$statement = $database->prepare($query);
	$statement->execute(@_);

	# Retreive and finish
	@values = $statement->fetchrow();
	$statement->finish();

	# Return retreived information
	return @values;
}

# Privileges
# Alias for Administrative  option
sub DataGetAdmin
{ return DataGet(shift, "SELECT PuppeteerAdminPrivs FROM Puppeteer WHERE PuppeteerLogin=?", @_); }

# Alias for Supervisory Option
sub DataGetSuper
{ return DataGet(shift, "SELECT PuppeteerSuperPrivs FROM Puppeteer WHERE PuppeteerLogin=?", @_); }

# Alias for Privileges
sub DataGetPrivs
{ return DataGet(shift, "SELECT PuppeteerSuperPrivs, PuppeteerAdminPrivs, PuppeteerModPrivs FROM Puppeteer WHERE PuppeteerLogin=?", @_); }

# Realm privs
sub DataGetRoomPrivs
{ return DataGet(shift, "SELECT p.PuppeteerLogin FROM Puppeteer p, RealmSupervisor r WHERE p.PuppeteerLogin=r.PuppeteerLogin AND r.RealmName=? AND p.PuppeteerLogin=? AND p.PuppeteerSuperPrivs='yes'", @_); }

# Moderator counts
sub DataGetModCount
{ return DataGet(shift, "SELECT COUNT(*) FROM RealmSupervisor WHERE PuppeteerLogin=?", @_); }


# Puppeteer
# Alias for Puppeteer Info
sub DataGetPuppeteerLogin
{ return DataGet(shift, "SELECT PuppeteerLogin FROM Puppet WHERE PuppetPool=? AND PuppetName=?", 'system', @_); }

# Alias for font settings
sub DataGetFont
{
	my ($text, $color, $face) = DataGet(shift, "SELECT PuppetTextSize, PuppetTextColour, PuppetTextFace FROM Puppet WHERE PuppetName=? AND PuppeteerLogin=?", shift, shift); 

	# Verify and define
	$text  = (defined($text))  ? $text  : 3;
	$color = (defined($color)) ? $color : shift;
	$face  = (defined($face))  ? $face  : shift;

	return ($text, $color, $face);
}

# Alias for frequency
sub DataGetFrenquency
{ return DataGet(shift, "SELECT COUNT(*) FROM Puppeteer WHERE PuppeteerTimestamp > ?", @_); }


# Puppeteer
# Alias for Javascript option
sub DataGetJavascript
{ return DataGet(shift, "SELECT PuppeteerJavascript FROM Puppeteer WHERE PuppeteerLogin=?", @_); }

# Alias for chat options
sub DataGetChatOptions
{ return DataGet(shift, "SELECT PuppeteerJavascript, PuppeteerChatPref FROM Puppeteer WHERE PuppeteerLogin=?", @_); }

# Alias for Formatting option
sub DataGetFormatting
{ return DataGet(shift, "SELECT PuppeteerFormatting FROM Puppeteer WHERE PuppeteerLogin=?", @_); }

# Alias for Supervisory option
sub DataGetBannedStatus
{ return DataGet(shift, "SELECT PuppeteerLogin FROM PuppeteerRestriction WHERE PuppeteerLogin=? AND RealmName=?", @_); }

# Alias for PuppeteerEmail retreival
sub DataGetPuppeteerEmail
{ return DataGet(shift, "SELECT PuppeteerEmail FROM Puppeteer WHERE PuppeteerLogin=?", @_); }

# Alias for Puppeteers Email
sub DataGetPuppeteerExistence
{ return DataGet(shift, "SELECT PuppeteerLogin FROM Puppeteer WHERE PuppeteerLogin=?", @_); }

# Alias for Puppeteers Homepage Expire Time
sub DataGetPuppeteerExpire
{ return DataGet(shift, "SELECT PuppeteerHomeSpan FROM Puppeteer WHERE PuppeteerLogin=?", @_); }

# Alias for Puppeteers Homepage
sub DataGetPuppeteerHome
{ return DataGet(shift, "SELECT PuppeteerHome FROM Puppeteer WHERE PuppeteerLogin=?", @_); }

# Alias for Mail retreival
sub DataGetPuppeteerMail
{ return DataGet(shift, "SELECT PuppeteerLogin, MailFrom, MailRcpt, MailSubject, MailTimestamp, MailContents FROM Mail WHERE MailIDNumber=?", @_); }

# Alias for Mail retreival
sub DataGetPuppeteerMailID
{ return DataGet(shift, "SELECT MailIDNumber FROM Mail WHERE PuppeteerLogin=? AND MailTimestamp=?", @_); }

# Alias for Name retreival
sub DataGetPuppeteerName
{ return DataGet(shift, "SELECT PuppeteerName FROM Puppeteer WHERE PuppeteerLogin=?", @_); }

# Alias for registration
sub DataGetPuppeteerReg
{ return DataGet(shift, "SELECT PuppeteerLogin FROM PuppeteerRegistration WHERE PuppeteerLogin=?", @_); }

# Puppets
# Default Puppet
sub DataGetDefault
{ 
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $default;						# Default puppet 


	#####################
	# Program area

	# Selected default
	($default) = DataGet($self, "SELECT PuppetName
		FROM Puppet
		WHERE PuppetDefault=?
		  AND PuppeteerLogin=?", 'yes', @_);

	# No default specified
	unless (defined($default))
	{
		# Alphebetical search
		($default) = DataGet($self, "SELECT PuppetName 
			FROM Puppet 
			WHERE PuppetPool=? 
			  AND PuppeteerLogin=? 
			ORDER BY PuppetName", 'system', @_); 
	}

	# Returns default handle
	return $default;
}

# Active
sub DataGetPuppetActive
{ return DataGet(shift, "SELECT PuppetActive FROM Puppet WHERE PuppetName=? AND PuppeteerLogin=?", @_); }


# Existence
sub DataGetPuppetExistence
{ return DataGet(shift, "SELECT PuppetName FROM Puppet WHERE PuppetName=? AND PuppeteerLogin=?", @_); }


# Realm
# Alias for Realm Access
sub DataGetRealmAccess
{ return DataGet(shift, "SELECT RealmAccess FROM Realm WHERE RealmName=?", @_); } 

# Alias for Alternate Message Boards
sub DataGetRealmAltBoard
{ return DataGet(shift, "SELECT RealmAltBoard FROM Realm WHERE RealmName=?", @_); }

# Alias for Realm Description
sub DataGetRealmDescription
{ return DataGet(shift, "SELECT DesignContent FROM RealmDesign WHERE WebchatTextName=? AND RealmName=?", 'RealmInfo', @_); }

# Alias for Realm Info
sub DataGetRealmInfo
{ return DataGet(shift, "SELECT RealmLanguage, RealmTags FROM Realm WHERE RealmName=?", @_); }

# Alias for Realm Image Info
sub DataGetRealmImageInfo
{ return DataGet(shift, "SELECT RealmImageHeight, RealmImageWidth, RealmImageSize FROM Realm WHERE RealmName=?", @_); }

# Alias for Realm existence
sub DataGetRealmExistence
{ return DataGet(shift, "SELECT RealmName, RealmPublic, RealmAlias FROM Realm WHERE RealmName=?", @_); }

# Alias for Realm password
sub DataGetRealmPassword
{ return DataGet(shift, "SELECT RealmPassword FROM Realm WHERE RealmName=?", @_); }

# Alias for Realm Public status
sub DataGetRealmPublic
{ return DataGet(shift, "SELECT RealmPublic FROM Realm WHERE RealmName=?", @_); }


# Misc
# Weather control
sub DataGetWeather
{ return DataGet(shift, "SELECT RealmMETAR FROM Realm WHERE RealmName=?", @_); }

# Fortunes
sub DataGetFortune
{ return DataGet(shift, "SELECT FortuneMsg FROM Fortune WHERE FortuneType=? ORDER BY RAND() LIMIT 1", @_); }


#####################
# Destructor
#
# Subroutine that will kick in once the object itself has passed away.  This
# will allow to automatically kill the database connection and this free
# us of adding that code

sub DESTROY
{
	#####################
	# Data Members

	my $self     = shift;					# Reference to self
	my $database = $self->{'HANDLE'};			# Database handle

	
	#####################
	# Program area

	# Disconnect from database
	$database->disconnect();
}

1;								# Return true
